@@ -84,7 +84,6 @@ static const uint8_t is_audio_command[10] = { 1, 1, 1, 1, 0, 0, 0, 1, 1, 0 };
 
 typedef struct ShortenContext {
     AVCodecContext *avctx;
-    AVFrame frame;
     GetBitContext gb;
 
     int min_framesize, max_framesize;
@@ -118,9 +117,6 @@ static av_cold int shorten_decode_init(AVCodecContext *avctx)
     ShortenContext *s = avctx->priv_data;
     s->avctx          = avctx;
 
-    avcodec_get_frame_defaults(&s->frame);
-    avctx->coded_frame = &s->frame;
-
     return 0;
 }
 
@@ -213,38 +209,34 @@ static int decode_wave_header(AVCodecContext *avctx, const uint8_t *header,
 {
     int len, bps;
     short wave_format;
-    GetByteContext gb;
+    const uint8_t *end= header + header_size;
 
-    bytestream2_init(&gb, header, header_size);
-
-    if (bytestream2_get_le32(&gb) != MKTAG('R', 'I', 'F', 'F')) {
+    if (bytestream_get_le32(&header) != MKTAG('R', 'I', 'F', 'F')) {
         av_log(avctx, AV_LOG_ERROR, "missing RIFF tag\n");
         return AVERROR_INVALIDDATA;
     }
 
-    bytestream2_skip(&gb, 4); /* chunk size */
+    header += 4; /* chunk size */
 
-    if (bytestream2_get_le32(&gb) != MKTAG('W', 'A', 'V', 'E')) {
+    if (bytestream_get_le32(&header) != MKTAG('W', 'A', 'V', 'E')) {
         av_log(avctx, AV_LOG_ERROR, "missing WAVE tag\n");
         return AVERROR_INVALIDDATA;
     }
 
-    while (bytestream2_get_le32(&gb) != MKTAG('f', 'm', 't', ' ')) {
-        len = bytestream2_get_le32(&gb);
-        bytestream2_skip(&gb, len);
-        if (len < 0 || bytestream2_get_bytes_left(&gb) < 16) {
-            av_log(avctx, AV_LOG_ERROR, "no fmt chunk found\n");
+    while (bytestream_get_le32(&header) != MKTAG('f', 'm', 't', ' ')) {
+        len     = bytestream_get_le32(&header);
+        if (len<0 || end - header - 8 < len)
             return AVERROR_INVALIDDATA;
-        }
+        header += len;
     }
-    len = bytestream2_get_le32(&gb);
+    len = bytestream_get_le32(&header);
 
     if (len < 16) {
         av_log(avctx, AV_LOG_ERROR, "fmt chunk was too short\n");
         return AVERROR_INVALIDDATA;
     }
 
-    wave_format = bytestream2_get_le16(&gb);
+    wave_format = bytestream_get_le16(&header);
 
     switch (wave_format) {
     case WAVE_FORMAT_PCM:
@@ -254,11 +246,11 @@ static int decode_wave_header(AVCodecContext *avctx, const uint8_t *header,
         return AVERROR(ENOSYS);
     }
 
-    bytestream2_skip(&gb, 2); // skip channels    (already got from shorten header)
-    avctx->sample_rate = bytestream2_get_le32(&gb);
-    bytestream2_skip(&gb, 4); // skip bit rate    (represents original uncompressed bit rate)
-    bytestream2_skip(&gb, 2); // skip block align (not needed)
-    bps = bytestream2_get_le16(&gb);
+    header += 2;        // skip channels    (already got from shorten header)
+    avctx->sample_rate = bytestream_get_le32(&header);
+    header += 4;        // skip bit rate    (represents original uncompressed bit rate)
+    header += 2;        // skip block align (not needed)
+    bps     = bytestream_get_le16(&header);
     avctx->bits_per_coded_sample = bps;
 
     if (bps != 16 && bps != 8) {
@@ -273,8 +265,7 @@ static int decode_wave_header(AVCodecContext *avctx, const uint8_t *header,
     return 0;
 }
 
-static const int fixed_coeffs[][3] = {
-    { 0,  0,  0 },
+static const int fixed_coeffs[3][3] = {
     { 1,  0,  0 },
     { 2, -1,  0 },
     { 3, -3,  1 }
@@ -303,12 +294,7 @@ static int decode_subframe_lpc(ShortenContext *s, int command, int channel,
     } else {
         /* fixed LPC coeffs */
         pred_order = command;
-        if (pred_order >= FF_ARRAY_ELEMS(fixed_coeffs)) {
-            av_log(s->avctx, AV_LOG_ERROR, "invalid pred_order %d\n",
-                   pred_order);
-            return AVERROR_INVALIDDATA;
-        }
-        coeffs     = fixed_coeffs[pred_order];
+        coeffs     = fixed_coeffs[pred_order - 1];
         qshift     = 0;
     }
 
@@ -426,6 +412,7 @@ static int read_header(ShortenContext *s)
 static int shorten_decode_frame(AVCodecContext *avctx, void *data,
                                 int *got_frame_ptr, AVPacket *avpkt)
 {
+    AVFrame *frame     = data;
     const uint8_t *buf = avpkt->data;
     int buf_size       = avpkt->size;
     ShortenContext *s  = avctx->priv_data;
@@ -437,7 +424,7 @@ static int shorten_decode_frame(AVCodecContext *avctx, void *data,
         void *tmp_ptr;
         s->max_framesize = 8192; // should hopefully be enough for the first header
         tmp_ptr = av_fast_realloc(s->bitstream, &s->allocated_bitstream_size,
-                                  s->max_framesize + FF_INPUT_BUFFER_PADDING_SIZE);
+                                  s->max_framesize);
         if (!tmp_ptr) {
             av_log(avctx, AV_LOG_ERROR, "error allocating bitstream buffer\n");
             return AVERROR(ENOMEM);
@@ -490,7 +477,7 @@ static int shorten_decode_frame(AVCodecContext *avctx, void *data,
 
     s->cur_chan = 0;
     while (s->cur_chan < s->channels) {
-        unsigned int cmd;
+        unsigned cmd;
         int len;
 
         if (get_bits_left(&s->gb) < 3 + FNSIZE) {
@@ -608,15 +595,15 @@ static int shorten_decode_frame(AVCodecContext *avctx, void *data,
                 int chan;
 
                 /* get output buffer */
-                s->frame.nb_samples = s->blocksize;
-                if ((ret = ff_get_buffer(avctx, &s->frame)) < 0) {
+                frame->nb_samples = s->blocksize;
+                if ((ret = ff_get_buffer(avctx, frame)) < 0) {
                     av_log(avctx, AV_LOG_ERROR, "get_buffer() failed\n");
                     return ret;
                 }
 
                 for (chan = 0; chan < s->channels; chan++) {
-                    samples_u8  = ((uint8_t **)s->frame.extended_data)[chan];
-                    samples_s16 = ((int16_t **)s->frame.extended_data)[chan];
+                    samples_u8  = ((uint8_t **)frame->extended_data)[chan];
+                    samples_s16 = ((int16_t **)frame->extended_data)[chan];
                     for (i = 0; i < s->blocksize; i++) {
                         switch (s->internal_ftype) {
                         case TYPE_U8:
@@ -630,9 +617,7 @@ static int shorten_decode_frame(AVCodecContext *avctx, void *data,
                     }
                 }
 
-
-                *got_frame_ptr   = 1;
-                *(AVFrame *)data = s->frame;
+                *got_frame_ptr = 1;
             }
         }
     }
